<?php
/*
Plugin Name: WP-ContentBlogroll
Plugin URI: http://www.delphiarea.com/products/wordpress-plugins/wp-contentblogroll/
Description: Defines a shortcode to list your blogroll in pages or posts.
Version: 1.0
Author: Kambiz R. Khojasteh
Author URI: kambiz@delphiarea.com
Tags: shortcode, blogroll, bookmarks, links, page, post
*/

add_shortcode('blogroll', 'blogroll_shortcode');
function blogroll_shortcode($atts) {
	$defaults = array(
		'categorized' => 1, 'category' => '', 'exclude_category' => '',	'category_name' => '', 
		'category_orderby' => 'name', 'category_order' => 'ASC', 'show_category_desc' => 1, 
		'include' => '', 'exclude' => '', 'orderby' => 'name', 'order' => 'ASC', 'limit' => -1, 
		'show_updated' => 0, 'show_images' => 0, 'show_desc' => 1, 'show_notes' => 1, 
		'show_rating' => 0, 'rating_image' => '', 'hide_invisible' => 1, 'none_message' => '');
	$atts = shortcode_atts($defaults, $atts);
	if ($atts['category'])
		$atts['category'] = link_category_slugs_to_ids($atts['category']);
	if ($atts['exclude_category'])
		$atts['exclude_category'] = link_category_slugs_to_ids($atts['exclude_category']);
	if ($atts['categorized']) {
		extract($atts, EXTR_SKIP);
		$cats = get_terms('link_category', array('name__like' => $category_name, 'include' => $category, 'exclude' => $exclude_category, 'orderby' => $category_orderby, 'order' => $category_order, 'hierarchical' => 0));
		$blogroll = '';
		foreach ((array) $cats as $cat) {
			$bookmarks = get_bookmarks_ex(array_merge($atts, array('category' => $cat->term_id)));
			if ($bookmarks) {
				$blogroll .= '<div id="blogroll-category-' . $cat->term_id . '" class="blogroll-category">';
				$blogroll .= '<h3 class="blogroll-category-name">' . attribute_escape($cat->name) . '</h3>';
				if ($show_category_desc && !empty($cat->description))
					$blogroll .= '<p class="blogroll-category-desc">' . attribute_escape($cat->description) . '</p>';
				$blogroll .= get_formated_bookmarks($bookmarks, $atts);
				$blogroll .= '</div>';
			}
		}
	}
	else {
		$bookmarks = get_bookmarks_ex($atts);
		if ($bookmarks)
			$blogroll = get_formated_bookmarks($bookmarks, $atts);
	}
	if (empty($blogroll) && $atts['none_message'])
		$blogroll = '<p class="blogroll_none">' . $atts['none_message'] . '</p>';
	if (!empty($blogroll))
		$blogroll = '<div class="blogroll">' . $blogroll . '</div>';
	return $blogroll;
}

function get_formated_bookmarks($bookmarks, $atts) {
	$defaults = array(
		'show_updated' => 0, 'show_images' => 0, 'show_desc' => 1, 
		'show_notes' => 0, 'show_rating' => 0, 'rating_image' => '');
	extract(shortcode_atts($defaults, $atts), EXTR_SKIP);
	
	if (!empty($rating_image) && strpos($image, 'http') === false)
		$rating_image = get_option('siteurl') . $rating_image;
	
	$out = '<ul class="blogroll-bookmarks">';
	
	foreach($bookmarks as $bookmark) {
	
		if (!isset($bookmark->recently_updated) )
			$bookmark->recently_updated = false;
		
		$name = attribute_escape($bookmark->link_name);
		$desc = attribute_escape($bookmark->link_description);
		
		$url = '#';
		if (!empty($bookmark->link_url));
			$url = clean_url($bookmark->link_url);
			
		$rel = $bookmark->link_rel;
		if (!empty($bookmark->link_rel))
			$rel = ' rel="' . $rel . '"';

		$target = $bookmark->link_target;
		if (!empty($bookmark->link_target))
			$target = ' target="' . $target . '"';
			
 		$title = '';
 		if (!$show_desc && !empty($desc))
 			$title = ' title="' . $desc . '"';

 		$image = '';
		if ($show_image && !empty($bookmark->link_image)) {
			$image = $bookmark->link_image;
			if (strpos($image, 'http') === false)
				$image = get_option('siteurl') . $image;
		}
		
		$star = '';
		if ($show_rating && !empty($bookmark->link_rating)) {
			if (!empty($rating_image))
				$star = '<img src="' . $rating_image . '" alt="" title="' . $bookmark->link_rating . ' / 9" />';
			else
				$star = '*';
		}
		
		$out .= '<li id="blogroll-bookmark-' . $bookmark->link_id . '">';
		
		$out .= '<h4  class="blogroll-bookmark-name">';
		if ($show_updated && $bookmark->recently_updated )
			$out .= get_option('links_recently_updated_prepend');
		$out .= '<a class="blogroll-bookmark-link" href="' . $url . '"' . $title . $target . $rel . '>';
		if ($show_image && !empty($image))
			$out .= '<img class="blogroll-bookmark-image" src="' . $image . '" alt="' . $name . '" />';
		else
			$out .= $name;
		$out .= '</a>';
		if ($show_updated && $bookmark->recently_updated)
			$output .= get_option('links_recently_updated_append');
		$out .= '</h4>';

		if ($show_updated && ('00' != substr($bookmark->link_updated_f, 0, 2)))
			$out .= '<small class="blogroll-bookmark-update">' . __('Last updated on') . ' ' . date_i18n(get_option('links_updated_date_format'), $bookmark->link_updated_f + (get_option('gmt_offset') * 3600)) . '</small>';
		
		if ($show_rating && !empty($star))
			$out .= '<div class="blogroll-bookmark-rating">' . str_repeat($star, (int) $bookmark->link_rating) . '</div>';

		if ($show_desc && !empty($desc))
			$out .= '<p class="blogroll-bookmark-desc">' . $desc . '</p>';
		if ($show_notes && !empty($bookmark->link_notes))
			$out .= '<div class="blogroll-bookmark-notes">' . apply_filters('the_content', $bookmark->link_notes) . '</div>';
			
		$out .= '</li>';
	}
	
	$out .= '</ul>';
	return $out;
}

function get_bookmarks_ex($params) {
	global $bookmarks_exclude_category;
	if (!empty($params['exclude_category']))
		$bookmarks_exclude_category = $params['exclude_category'];
	else
		$bookmarks_exclude_category = '';
	add_action('query', 'fix_get_bookmarks_query');
	$result = get_bookmarks($params);
	remove_action('query', 'fix_get_bookmarks_query');
	return $result;
}

function fix_get_bookmarks_query($query) {
	global $bookmarks_exclude_category, $wpdb;
	if (!empty($bookmarks_exclude_category)) {
		// if a bookmark is in multiple categories and one of them is excluded, ignore the bookmark
		$subquery =  "SELECT s_l.link_id FROM $wpdb->links as s_l INNER JOIN $wpdb->term_relationships AS s_tr ON s_l.link_id = s_tr.object_id INNER JOIN $wpdb->term_taxonomy as s_tt ON s_tt.term_taxonomy_id = s_tr.term_taxonomy_id WHERE s_l.link_id = wp_links.link_id AND s_tt.taxonomy = 'link_category' AND s_tt.term_id IN ($bookmarks_exclude_category)";
		$query = str_replace(' ORDER BY ', " AND NOT EXISTS($subquery) ORDER BY ", $query);
	}
	return $query;
}

function link_category_slugs_to_ids($categories) {
	$categories = preg_split('/[\s,]+/', $categories);
	foreach($categories as $i => $category) {
		if (!is_numeric($category)) {
			$term = get_term_by('slug', $category, 'link_category');
			if ($term)
				$categories[$i]	= $term->term_id;
		}
	}
	return implode(',', $categories);
}
  
?>